@@ -55,8 +55,8 @@ guides_build <- function(guides, theme) {
   legend.spacing.x <- theme$legend.spacing.x  %||% legend.spacing
   legend.box.margin <- theme$legend.box.margin %||% margin()
 
-  widths <- do.call(unit.c, lapply(guides, gtable_width))
-  heights <- do.call(unit.c, lapply(guides, gtable_height))
+  widths <- exec(unit.c, !!!lapply(guides, gtable_width))
+  heights <- exec(unit.c, !!!lapply(guides, gtable_height))
 
   just <- valid.just(theme$legend.box.just)
   xjust <- just[1]
@@ -142,8 +142,9 @@ assemble_guides <- function(guides, theme) {
 #' @importFrom gtable gtable_width gtable_height
 #' @importFrom grid unit.c
 attach_guides <- function(table, guides, theme) {
-  if (any(table$layout$name == 'panel-guide_area')) {
-    area_ind <- which(table$layout$name == 'panel-guide_area')
+  guide_areas <- grepl('panel-guide_area', table$layout$name)
+  if (any(guide_areas)) {
+    area_ind <- which(guide_areas)
     if (length(area_ind) != 1) {
       warning("Only using the first guide area", call. = FALSE)
     }
