@@ -22,26 +22,50 @@ print.patchwork <- function(x, newpage = is.null(vp), vp = NULL, ...) {
 
   set_last_plot(x)
 
-  if (is.null(vp)) {
-    grid.draw(gtable)
-  } else {
+  if (!is.null(vp)) {
     if (is.character(vp)) {
       seekViewport(vp)
     } else {
       pushViewport(vp)
     }
-    grid.draw(gtable)
+  }
+
+  tryCatch(
+    grid.draw(gtable),
+    error = function(e) {
+      if (inherits(e, 'simpleError') && deparse(conditionCall(e)[[1]]) == 'grid.Call') {
+        if (Sys.getenv("RSTUDIO") == "1") {
+          cli_abort(c("The RStudio {.field Plots} window may be too small to show this patchwork.",
+                    i = "Please make the window larger.")
+          )
+        } else {
+          cli_abort(c("The viewport may be too small to show this patchwork.",
+                      i = "Please make the window larger.")
+          )
+        }
+      }
+    }
+  )
+
+  if (!is.null(vp)) {
     upViewport()
   }
+
   invisible(x)
 }
 #' @export
 plot.patchwork <- print.patchwork
 #' @export
+length.patchwork <- function(x) {
+  length(x$patches$plots) + !is_empty(x)
+}
+#' @export
+names.patchwork <- function(x) NULL
+#' @export
 `[[.patchwork` <- function(x, ..., exact = TRUE) {
   ind <- ..1
   if (!is.numeric(ind)) {
-    stop('Patchworks can only be indexed with numeric indices', call. = FALSE)
+    cli_abort('Patchworks can only be indexed with numeric indices')
   }
 
   n_patches <- length(x$patches$plots)
@@ -51,13 +75,13 @@ plot.patchwork <- print.patchwork
     class(plot) <- setdiff(class(plot), 'patchwork')
   } else {
     if (ind > n_patches) {
-      stop('Index out of bounds', call. = FALSE)
+      cli_abort('Index out of bounds')
     }
     plot <- x$patches$plots[[ind[1]]]
   }
   if (length(ind) > 1) {
     if (!is_patchwork(plot)) {
-      stop('Can only do nested indexing into patchworks', call. = FALSE)
+      cli_abort('Can only do nested indexing into patchworks')
     }
     plot <- plot[[ind[-1]]]
   }
@@ -67,7 +91,7 @@ plot.patchwork <- print.patchwork
 `[[<-.patchwork` <- function(x, ..., value) {
   ind <- ..1
   if (!is.numeric(ind)) {
-    stop('Patchworks can only be indexed with numeric indices', call. = FALSE)
+    cli_abort('Patchworks can only be indexed with numeric indices')
   }
 
   if (!is.ggplot(value)) {
@@ -76,13 +100,13 @@ plot.patchwork <- print.patchwork
   n_patches <- length(x$patches$plots)
   if (!is_empty(x) && ind == n_patches + 1) {
     if (length(ind) != 1) {
-      stop('Can only do nested indexing into patchworks', call. = FALSE)
+      cli_abort('Can only do nested indexing into patchworks')
     }
     return(add_patches(value, x$patches))
   }
   if (length(ind) > 1) {
     if (!is_patchwork(x$patches$plots[[ind[1]]])) {
-      stop('Can only do nested indexing into patchworks', call. = FALSE)
+      cli_abort('Can only do nested indexing into patchworks')
     }
     x$patches$plots[[ind[1]]][[ind[-1]]] <- value
   } else {
@@ -90,6 +114,30 @@ plot.patchwork <- print.patchwork
   }
   x
 }
+#' @importFrom utils str
+#' @export
+str.patchwork <- function(object, ...) {
+  n_patches <- length(object$patches$plots)
+  if (!is_empty(object)) n_patches <- n_patches + 1
+  cat('A patchwork composed of ', n_patches, ' patches\n', sep = '')
+  cat('- Autotagging is turned ', if (is.null(object$patches$annotation$tag_levels)) 'off' else 'on', '\n', sep = '')
+  cat('- Guides are ', if (isTRUE(object$patches$layout$guides == 'collect')) 'collected' else 'kept', '\n', sep = '')
+  cat('\n')
+  cat('Layout:\n')
+  if (is.null(object$layout$design)) {
+    l <- object$layout
+    if (is.null(l$ncol) && !is.null(l$widths) && length(l$widths) > 1) {
+      l$ncol <- length(l$widths)
+    }
+    if (is.null(l$nrow) && !is.null(l$heights) && length(l$heights) > 1) {
+      l$nrow <- length(l$heights)
+    }
+    dims <- wrap_dims(n_patches, nrow = l$nrow, ncol = l$ncol)
+    print(create_design(dims[2], dims[1], isTRUE(l$byrow)))
+  } else {
+    print(object$layout$design)
+  }
+}
 #' @importFrom ggplot2 ggplot_build ggplot_gtable panel_rows panel_cols wrap_dims
 #' @importFrom gtable gtable
 #' @importFrom grid unit unit.pmax is.unit
@@ -104,10 +152,10 @@ build_patchwork <- function(x, guides = 'auto') {
     x$layout$guides
   }
   gt <- lapply(x$plots, plot_table, guides = guides)
-  fixed_asp <- vapply(gt, function(x) isTRUE(x$respect), logical(1))
   guide_grobs <- unlist(lapply(gt, `[[`, 'collected_guides'), recursive = FALSE)
   gt <- lapply(gt, simplify_gt)
   gt <- add_insets(gt)
+  fixed_asp <- vapply(gt, function(x) isTRUE(x$respect), logical(1))
   if (is.null(x$layout$design)) {
     if (is.null(x$layout$ncol) && !is.null(x$layout$widths) && length(x$layout$widths) > 1) {
       x$layout$ncol <- length(x$layout$widths)
@@ -116,7 +164,7 @@ build_patchwork <- function(x, guides = 'auto') {
       x$layout$nrow <- length(x$layout$heights)
     }
     dims <- wrap_dims(length(gt), nrow = x$layout$nrow, ncol = x$layout$ncol)
-    x$layout$design <- create_design(dims[2], dims[1], x$layout$byrow)
+    x$layout$design <- create_design(dims[2], dims[1], isTRUE(x$layout$byrow))
   } else {
     dims <- c(
       max(x$layout$design$b),
@@ -139,7 +187,7 @@ build_patchwork <- function(x, guides = 'auto') {
   if (any(design$r > dims[2])) design$r[design$r > dims[2]] <- dims[2]
   max_z <- lapply(gt, function(x) max(x$layout$z))
   max_z <- c(0, cumsum(max_z))
-  gt_new$layout <- do.call(rbind, lapply(seq_along(gt), function(i) {
+  gt_new$layout <- exec(rbind, !!!lapply(seq_along(gt), function(i) {
     loc <- design[i, ]
     lay <- gt[[i]]$layout
     lay$name <- paste0(lay$name, '-', i)
@@ -177,6 +225,16 @@ build_patchwork <- function(x, guides = 'auto') {
     gt_new$collected_guides <- guide_grobs
   }
 
+  gt_new <- gtable_add_grob(
+    gt_new, zeroGrob(),
+    t = PANEL_ROW,
+    l = PANEL_COL,
+    b = PANEL_ROW + TABLE_ROWS * (dims[1] - 1),
+    r = PANEL_COL + TABLE_COLS * (dims[2] - 1),
+    z = -1,
+    name = "panel-area"
+  )
+
   class(gt_new) <- c('gtable_patchwork', class(gt_new))
   gt_new
 }
@@ -305,7 +363,7 @@ simplify_free <- function(gt, gt_new, panels, rows, cols) {
     if (any(t_strips)) {
       gt_new$grobs[t_strips] <- lapply(gt_new$grobs[t_strips], function(g) {
         if (is.gtable(g)) {
-          g$vp <- viewport(y = 0, just = 'bottom', height = g$heights)
+          g$vp <- viewport(y = 0, just = 'bottom', height = sum(g$heights))
         }
         g
       })
@@ -314,7 +372,7 @@ simplify_free <- function(gt, gt_new, panels, rows, cols) {
     if (any(b_strips)) {
       gt_new$grobs[b_strips] <- lapply(gt_new$grobs[b_strips], function(g) {
         if (is.gtable(g)) {
-          g$vp <- viewport(y = 1, just = 'top', height = g$heights)
+          g$vp <- viewport(y = 1, just = 'top', height = sum(g$heights))
         }
         g
       })
@@ -358,7 +416,7 @@ simplify_free <- function(gt, gt_new, panels, rows, cols) {
     if (any(l_strips)) {
       gt_new$grobs[l_strips] <- lapply(gt_new$grobs[l_strips], function(g) {
         if (is.gtable(g)) {
-          g$vp <- viewport(x = 1, just = 'right', width = g$widths)
+          g$vp <- viewport(x = 1, just = 'right', width = sum(g$widths))
         }
         g
       })
@@ -367,7 +425,7 @@ simplify_free <- function(gt, gt_new, panels, rows, cols) {
     if (any(r_strips)) {
       gt_new$grobs[r_strips] <- lapply(gt_new$grobs[r_strips], function(g) {
         if (is.gtable(g)) {
-          g$vp <- viewport(x = 0, just = 'left', width = g$widths)
+          g$vp <- viewport(x = 0, just = 'left', width = sum(g$widths))
         }
         g
       })
@@ -595,19 +653,21 @@ add_strips <- function(gt) {
   )
   if (!any(grepl('strip-b', gt$layout$name))) {
     gt <- gtable_add_rows(gt, unit(0, 'mm'), panel_loc$b + strip_pos)
-  } else if (strip_pos == 2) {
+  } else if (strip_pos == 2 && !any(gt$layout$b == panel_loc$b + 2)) {
+    # Merge the strip-gap height into the axis and remove it. Only performed if
+    # an axis exist
     gt$heights[panel_loc$b + 1] <- sum(gt$heights[panel_loc$b + c(1, 2)])
     gt <- gt[-(panel_loc$b + 2), ]
   }
   if (!any(grepl('strip-t', gt$layout$name))) {
     gt <- gtable_add_rows(gt, unit(0, 'mm'), panel_loc$t - 1 - strip_pos)
-  } else if (strip_pos == 2) {
+  } else if (strip_pos == 2 && !any(gt$layout$t == panel_loc$t - 2)) {
     gt$heights[panel_loc$t - 1] <- sum(gt$heights[panel_loc$t - c(1, 2)])
     gt <- gt[-(panel_loc$t - 2), ]
   }
   if (!any(grepl('strip-r', gt$layout$name))) {
     gt <- gtable_add_cols(gt, unit(0, 'mm'), panel_loc$r + strip_pos)
-  } else if (strip_pos == 2) {
+  } else if (strip_pos == 2 && !any(gt$layout$r == panel_loc$r + 2)) {
     gt$widths[panel_loc$r + 1] <- sum(gt$widths[panel_loc$r + c(1, 2)])
     gt <- gt[, -(panel_loc$r + 2)]
   }
@@ -726,8 +786,8 @@ set_panel_dimensions <- function(gt, panels, widths, heights, fixed_asp, design)
       panel_ind <- grep('panel', panels[[fixed_gt[i]]]$layout$name)[1]
       w <- panels[[fixed_gt[i]]]$grobs[[panel_ind]]$widths
       h <- panels[[fixed_gt[i]]]$grobs[[panel_ind]]$heights
-      can_set_width <- width_strings[fixed_areas[[i]]$cols] == '-1null' && length(w) == 1 && length(h) == 1
-      can_set_height <- height_strings[fixed_areas[[i]]$rows] == '-1null' && length(w) == 1 && length(h) == 1
+      can_set_width <- all(width_strings[fixed_areas[[i]]$cols] == '-1null') && length(w) == 1 && length(h) == 1
+      can_set_height <- all(height_strings[fixed_areas[[i]]$rows] == '-1null') && length(w) == 1 && length(h) == 1
       will_be_fixed <- TRUE
       if (can_set_width && can_set_height) {
         widths[fixed_areas[[i]]$cols] <- w
@@ -764,7 +824,7 @@ add_insets <- function(gt) {
   }
   canvas <- rank(cumsum(!is_inset), ties.method = "min")[is_inset]
   if (canvas[1] == 0) {
-    stop("insets cannot be the first plot in a patchwork", call. = FALSE)
+    cli_abort("insets cannot be the first plot in a patchwork")
   }
   insets <- which(is_inset)
   name <- paste0('inset_', insets)
@@ -789,7 +849,7 @@ add_insets <- function(gt) {
                                   PLOT_RIGHT, z = z, clip =  setting$clip, name = name[i]),
            full = gtable_add_grob(can, list(ins), 1, 1, nrow(can), ncol(can), z = z,
                                   clip = setting$clip, name = name[i]),
-           stop('Unknown alignment setting: `', setting$align_to, '`', call. = FALSE)
+           cli_abort('Unknown alignment setting: {.arg {setting$align_to}}')
     )
   }
   gt[!is_inset]
